
#include "Slew.hpp"

bool Slew::active() {
	return inputs[IN_INPUT].isConnected() && outputs[OUT_OUTPUT].isConnected();
}

int Slew::channels() {
	return inputs[IN_INPUT].getChannels();
}

void Slew::modulateChannel(int c) {
	_slew[c].modulate(
		APP->engine->getSampleRate(),
		params[RISE_PARAM],
		&inputs[RISE_INPUT],
		10000.0f,
		params[RISE_SHAPE_PARAM],
		params[FALL_PARAM],
		&inputs[FALL_INPUT],
		10000.0f,
		params[FALL_SHAPE_PARAM],
		c
	);
}

void Slew::processChannel(const ProcessArgs& args, int c) {
	outputs[OUT_OUTPUT].setChannels(_channels);
	outputs[OUT_OUTPUT].setVoltage(_slew[c].next(inputs[IN_INPUT].getPolyVoltage(c)), c);
}

struct SlewWidget : BGModuleWidget {
	static constexpr int hp = 3;

	SlewWidget(Slew* module) {
		setModule(module);
		box.size = Vec(RACK_GRID_WIDTH * hp, RACK_GRID_HEIGHT);
		setPanel(box.size, "Slew");
		createScrews();

		// generated by svg_widgets.rb
		auto riseParamPosition = Vec(9.5, 34.0);
		auto riseShapeParamPosition = Vec(14.5, 76.0);
		auto fallParamPosition = Vec(9.5, 155.0);
		auto fallShapeParamPosition = Vec(14.5, 197.0);

		auto riseInputPosition = Vec(10.5, 105.0);
		auto fallInputPosition = Vec(10.5, 226.0);
		auto inInputPosition = Vec(10.5, 263.0);

		auto outOutputPosition = Vec(10.5, 301.0);
		// end generated by svg_widgets.rb

		addParam(createParam<Knob26>(riseParamPosition, module, Slew::RISE_PARAM));
		addParam(createParam<Knob16>(riseShapeParamPosition, module, Slew::RISE_SHAPE_PARAM));
		addParam(createParam<Knob26>(fallParamPosition, module, Slew::FALL_PARAM));
		addParam(createParam<Knob16>(fallShapeParamPosition, module, Slew::FALL_SHAPE_PARAM));

		addInput(createInput<Port24>(riseInputPosition, module, Slew::RISE_INPUT));
		addInput(createInput<Port24>(fallInputPosition, module, Slew::FALL_INPUT));
		addInput(createInput<Port24>(inInputPosition, module, Slew::IN_INPUT));

		addOutput(createOutput<Port24>(outOutputPosition, module, Slew::OUT_OUTPUT));
	}
};

Model* modelSlew = bogaudio::createModel<Slew, SlewWidget>("Bogaudio-Slew", "SLEW", "Slew limiter / lag generator / glide", "Slew limiter", "Polyphonic");
